/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml.Serialization;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;
using System.Runtime.InteropServices;
using System.Web.UI.HtmlControls;
using Microsoft.SharePoint.WebControls; // for SPControl
using PREPWorkflow;
using System.Collections;
using System.Configuration;


namespace PREPWebPartLibrary20
{
	/// <summary>
	/// The PREP Project Administration Web Part. It allows an Administrator to create, update, and delete PREP Assignments. 
	/// It also allows an Administrator to add or remove users from an assignment. When this happens, it handles 
	/// creation and deletion of PREP tasks associated with these users. 
	/// Behind the scenes, It writes to the Events list to administer PREP Assignments, which are defined 
	/// as three Events: Assigmnent Given, Ideas Due, and Reviews Due. 
	/// To administer students, it creates Custom SharePoint Roles, and adds or removes students from them.
	/// </summary>
	[DefaultProperty("Text"),
		ToolboxData("<{0}:PREPProjectAdmin20 runat=server></{0}:PREPProjectAdmin20>"),
		XmlRoot(Namespace="PREPWebPartLibrary20")]
	public class PREPProjectAdmin20 : WebPart
	{
		Label lblMessage;

		Label lblTitle;

		Button btnNew;
		
		TextBox txtAssignment;
		RequiredFieldValidator rfvAssignment;
		RegularExpressionValidator regExAssignment;

		TextBox txtDescription;
		RequiredFieldValidator rfvDescription;

		TextBox txtAssignmentDate;
		RequiredFieldValidator rfvAssignmentDate;
		RegularExpressionValidator regExAssignmentDate;

		TextBox txtIdeasDueDate;
		RequiredFieldValidator rfvIdeasDueDate;
		RegularExpressionValidator regExIdeasDueDate;

		TextBox txtReviewsDueDate;
		RequiredFieldValidator rfvReviewsDueDate;
		RegularExpressionValidator regExReviewsDueDate;

		ValidationSummary validationSummary;
		
		Button btnSubmit;

		Repeater rptAssignments;

		ArrayList prepAssignments;

		Label lblUserTitle;
		CheckBoxList cblUsers;
		Button btnUsers;
		
		SPSite PREPSite;
		SPWeb PREPWeb;

		private string prepTaskListName = ConfigurationSettings.AppSettings["PREPWorkflow.TaskList"];

		/// <summary>
		/// Override OnInit in order to set the title of the web part
		/// </summary>
		/// <param name="e"></param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit (e);
			this.Title = "PREP Administration";
		}
		
		/// <summary>
		/// Method which creates the ASP.NET controls on a Web Part. This process is 
		/// much more manual and code-intensive than it is on a regular ASP.NET page.
		/// </summary>
		protected override void CreateChildControls()
		{
			
			// Initialize objects
			PREPSite = SPControl.GetContextSite(Context);
			PREPWeb = SPControl.GetContextWeb(Context);
			prepAssignments = PREPAssignment.PREPAssignment_Collection(PREPWeb);

			// Title
			lblTitle = new Label();
			lblTitle.Text = HTMLTools.Literal("<b>New PREP Assignment Creation</b>");

			// New Assignment button - clears the form
			btnNew = new Button();
			btnNew.CausesValidation = false;
			btnNew.Text = "New Assignment";
			btnNew.Click +=new EventHandler(btnNew_Click);
			Controls.Add(btnNew);
			
			// Error / Completion Message
			lblMessage = new Label();
			lblMessage.Visible = false;
			lblMessage.ID = "lblMessageControl";
			
			// Assignment Name TextBox
			txtAssignment = new TextBox();
			txtAssignment.Width = 200;
			txtAssignment.ID = "txtAssignmentControl";
			Controls.Add(txtAssignment);

			// Assignment name text box Required Field Validator
			rfvAssignment = new RequiredFieldValidator();
			rfvAssignment.Text = " *";
			rfvAssignment.ErrorMessage = "- Assignment Name is required.";
			rfvAssignment.Display = ValidatorDisplay.Static;
			rfvAssignment.ControlToValidate = "txtAssignmentControl";
			Controls.Add(rfvAssignment);

			// Assignment name Reqular Expression Validator
			regExAssignment = new RegularExpressionValidator();
			regExAssignment.Text = " *";
			regExAssignment.ID = "regExAssignmentControl";
			regExAssignment.ErrorMessage = "- Assignment Name cannot contain spaces.";
			regExAssignment.Display = ValidatorDisplay.Static;
			regExAssignment.ControlToValidate = "txtAssignmentControl";
			//Any alpha or numeric character, no spaces
			//regExAssignment.ValidationExpression = "^[a-zA-Z0-9]+$";
			
			// this expression will accept any ascii character in the range of ! to ~
			// In other words, all of them. (! is lowest, ~ is highest). Will not accept spaces.
			regExAssignment.ValidationExpression = "^[!-~]+$";
			//<asp:regularexpressionvalidator id="RegularExpressionValidator1" runat="server" ErrorMessage="- Assignment Given date is not a valid date."
			//Display="Dynamic" ControlToValidate="txtAssignmentDate" ValidationExpression="^(?:(?:(?:0?[13578]|1[02])(\/|-)31)|(?:(?:0?[1,3-9]|1[0-2])(\/|-)(?:29|30)))(\/|-)(?:[1-9]\d\d\d|\d[1-9]\d\d|\d\d[1-9]\d|\d\d\d[1-9])$|^(?:(?:0?[1-9]|1[0-2])(\/|-)(?:0?[1-9]|1\d|2[0-8]))(\/|-)(?:[1-9]\d\d\d|\d[1-9]\d\d|\d\d[1-9]\d|\d\d\d[1-9])$|^(0?2(\/|-)29)(\/|-)(?:(?:0[48]00|[13579][26]00|[2468][048]00)|(?:\d\d)?(?:0[48]|[2468][048]|[13579][26]))$ ">*</asp:regularexpressionvalidator>
			Controls.Add(regExAssignment);

			// Description TextBox
			txtDescription = new TextBox();
			txtDescription.Columns = 35;
			txtDescription.Rows = 10;
			txtDescription.TextMode = TextBoxMode.MultiLine;
			txtDescription.ID = "txtDescriptionControl";
			Controls.Add(txtDescription);

			// Description TextBox Required Field Validator
			rfvDescription = new RequiredFieldValidator();
			rfvDescription.Text = " *";
			rfvDescription.ErrorMessage = "- Description is required.";
			rfvDescription.Display = ValidatorDisplay.Dynamic;
			rfvDescription.ControlToValidate = "txtDescriptionControl";
			Controls.Add(rfvDescription);

			//<asp:regularexpressionvalidator id="RegularExpressionValidator1" runat="server" ErrorMessage="- Assignment Given date is not a valid date."
			//Display="Dynamic" ControlToValidate="txtAssignmentDate" ValidationExpression="^(?:(?:(?:0?[13578]|1[02])(\/|-)31)|(?:(?:0?[1,3-9]|1[0-2])(\/|-)(?:29|30)))(\/|-)(?:[1-9]\d\d\d|\d[1-9]\d\d|\d\d[1-9]\d|\d\d\d[1-9])$|^(?:(?:0?[1-9]|1[0-2])(\/|-)(?:0?[1-9]|1\d|2[0-8]))(\/|-)(?:[1-9]\d\d\d|\d[1-9]\d\d|\d\d[1-9]\d|\d\d\d[1-9])$|^(0?2(\/|-)29)(\/|-)(?:(?:0[48]00|[13579][26]00|[2468][048]00)|(?:\d\d)?(?:0[48]|[2468][048]|[13579][26]))$ ">*</asp:regularexpressionvalidator>

			// Assignment Given Date TextBox
			txtAssignmentDate = new TextBox();
			txtAssignmentDate.MaxLength = 200;
			txtAssignmentDate.ID = "txtAssignmentDateControl";
			Controls.Add(txtAssignmentDate);

			// Assignment name text box Required Field Validator
			rfvAssignmentDate = new RequiredFieldValidator();
			rfvAssignmentDate.Text = " *";
			rfvAssignmentDate.ErrorMessage = "- Assignment Given Date is required.";
			rfvAssignmentDate.Display = ValidatorDisplay.Static;
			rfvAssignmentDate.ControlToValidate = "txtAssignmentDateControl";
			Controls.Add(rfvAssignmentDate);

			// Assignment name Reqular Expression Validator
			regExAssignmentDate = new RegularExpressionValidator();
			regExAssignmentDate.Text = " *";
			regExAssignmentDate.ID = "regExAssignmentDateControl";
			regExAssignmentDate.ErrorMessage = "- Assignment Given date is not a valid date.";
			regExAssignmentDate.Display = ValidatorDisplay.Static;
			regExAssignmentDate.ControlToValidate = "txtAssignmentDateControl";
			
			// Date Validation RegEx for mm-dd-yyyy.
			regExAssignmentDate.ValidationExpression = @"^(?:(?:(?:0?[13578]|1[02])(\\/|-)31)|(?:(?:0?[1,3-9]|1[0-2])(\\/|-)(?:29|30)))(\/|-)(?:[1-9]\d\d\d|\d[1-9]\d\d|\d\d[1-9]\d|\d\d\d[1-9])$|^(?:(?:0?[1-9]|1[0-2])(\/|-)(?:0?[1-9]|1\d|2[0-8]))(\/|-)(?:[1-9]\d\d\d|\d[1-9]\d\d|\d\d[1-9]\d|\d\d\d[1-9])$|^(0?2(\/|-)29)(\/|-)(?:(?:0[48]00|[13579][26]00|[2468][048]00)|(?:\d\d)?(?:0[48]|[2468][048]|[13579][26]))$ ";
			Controls.Add(regExAssignmentDate);

			// Ideas Due Date TextBox
			txtIdeasDueDate = new TextBox();
			txtIdeasDueDate.Width = 25;
			txtIdeasDueDate.ID = "txtIdeasDueDateControl";
			Controls.Add(txtIdeasDueDate);

			// Ideas Due Date Required Field Validator
			rfvIdeasDueDate = new RequiredFieldValidator();
			rfvIdeasDueDate.Text = " *";
			rfvIdeasDueDate.ErrorMessage = "- Ideas Due Date is required.";
			rfvIdeasDueDate.Display = ValidatorDisplay.Static;
			rfvIdeasDueDate.ControlToValidate = "txtIdeasDueDateControl";
			Controls.Add(rfvIdeasDueDate);

			// Ideas Due Date Reqular Expression Validator
			regExIdeasDueDate = new RegularExpressionValidator();
			regExIdeasDueDate.Text = " *";
			regExIdeasDueDate.ID = "regExIdeasDueDateControl";
			regExIdeasDueDate.ErrorMessage = "- Ideas Due Date number must be 1-3 digits only.";
			regExIdeasDueDate.Display = ValidatorDisplay.Static;
			regExIdeasDueDate.ControlToValidate = "txtIdeasDueDateControl";
			
			// this expression will accept only digits, and no more than 3 digits.
			regExIdeasDueDate.ValidationExpression = @"\d{1,3}";
			Controls.Add(regExIdeasDueDate);

			// Reviews Due Date TextBox
			txtReviewsDueDate = new TextBox();
			txtReviewsDueDate.Width = 25;
			txtReviewsDueDate.ID = "txtReviewsDueDateControl";
			Controls.Add(txtReviewsDueDate);

			// Reviews Due Date Required Field Validator
			rfvReviewsDueDate = new RequiredFieldValidator();
			rfvReviewsDueDate.Text = " *";
			rfvReviewsDueDate.ErrorMessage = "- Reviews Due Date is required.";
			rfvReviewsDueDate.Display = ValidatorDisplay.Static;
			rfvReviewsDueDate.ControlToValidate = "txtReviewsDueDateControl";
			Controls.Add(rfvReviewsDueDate);

			// Reviews Due Date Reqular Expression Validator
			regExReviewsDueDate = new RegularExpressionValidator();
			regExReviewsDueDate.Text = " *";
			regExReviewsDueDate.ID = "regExReviewsDueDateControl";
			regExReviewsDueDate.ErrorMessage = "- Reviews Due number must be 1-3 digits only.";
			regExReviewsDueDate.Display = ValidatorDisplay.Static;
			regExReviewsDueDate.ControlToValidate = "txtReviewsDueDateControl";
			
			// this expression will accept only digits, and no more than 3 digits.
			regExReviewsDueDate.ValidationExpression = @"\d{1,3}";
			Controls.Add(regExReviewsDueDate);
		
			// Validation Summary
			validationSummary = new ValidationSummary();
			validationSummary.Width = Unit.Pixel(300);
			validationSummary.DisplayMode = ValidationSummaryDisplayMode.List;
			validationSummary.ID = "validationSummaryControl";
			Controls.Add(validationSummary);
						
			// Submit button
			btnSubmit = new Button();
			btnSubmit.CausesValidation = true;
			btnSubmit.Text = "Create Assignment";
			btnSubmit.Enabled = true;
			btnSubmit.Click += new EventHandler(btnSubmit_Click);
			Controls.Add(btnSubmit);

			// Assignment Repeater
			rptAssignments = new Repeater();
			rptAssignments.HeaderTemplate = new RepeaterHeader();
			rptAssignments.ItemTemplate = new RepeaterColumn();
			rptAssignments.AlternatingItemTemplate = new RepeaterColumn();

			rptAssignments.FooterTemplate = new RepeaterFooter();
			rptAssignments.ItemCommand +=new RepeaterCommandEventHandler(rptAssignments_ItemCommand);
			rptAssignments.DataSource = prepAssignments;
			rptAssignments.DataBind();
			Controls.Add(rptAssignments);

			// Title
			lblUserTitle = new Label();
			lblUserTitle.Text = HTMLTools.Literal("<b>Students for this Assignment</b>");

			// User CheckBoxList
			cblUsers = new CheckBoxList(); //CustomCheckBoxList(); //CheckBoxList();
			cblUsers.ID = "cblUsersControl";
			cblUsers.EnableViewState = true;

			if(Page.IsPostBack)
			{
				LoadCheckBoxListOnPostBack();
			}

			// Users button - submits changes to user collection
			btnUsers = new Button();
			btnUsers.CausesValidation = false;
			btnUsers.Text = "Submit Student Changes";
			btnUsers.Click +=new EventHandler(btnUsers_Click);
			Controls.Add(btnUsers);
		}

		/// <summary>
		/// Writes the Web Part to the HTML output stream.
		/// </summary>
		/// <param name="output"></param>
		protected override void RenderWebPart(HtmlTextWriter output)
		{
			EnsureChildControls();
			
			output.Write(Environment.NewLine);
			output.Write(HTMLTools.Comment("-=-=-= Begin PREP Project Administration =-=-=-"));

			output.Write(HTMLTools.StartTable());

			// Assigmments Repeater
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colSpan=\"2\""));
			rptAssignments.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Title row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			lblTitle.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Message row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			output.Write(HTMLTools.StartSpan("style=\"font-color:#CCCCCC; font-weight:bold;\""));
			lblMessage.RenderControl(output);
			output.Write(HTMLTools.EndSpan());
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Assignment Name Row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol());
			output.Write(HTMLTools.Literal("Assignment Name"));
			rfvAssignment.RenderControl(output);
			regExAssignment.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.StartCol());
			txtAssignment.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Description Rows (2 rows)
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol());
			output.Write("Description");
			rfvDescription.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.StartCol());
			output.Write(HTMLTools.EmptyLine());
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			txtDescription.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Assignment Given Date Row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol());
			output.Write("Assignment Given Date:");
			rfvAssignmentDate.RenderControl(output);
			regExAssignmentDate.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.StartCol());
			txtAssignmentDate.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			output.Write(HTMLTools.Literal("(Use mm-dd-yyyy format)."));
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Ideas Due Date Row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol());
			output.Write("Ideas Due Date:");
			rfvIdeasDueDate.RenderControl(output);
			regExIdeasDueDate.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.StartCol());
			txtIdeasDueDate.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			output.Write(HTMLTools.Literal("(Number of days after Assignment Given)"));
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Reviews Due Date Row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol());
			output.Write("Reviews Due Date:");
			rfvReviewsDueDate.RenderControl(output);
			regExReviewsDueDate.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.StartCol());
			txtReviewsDueDate.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			output.Write(HTMLTools.Literal("(Number of days after Assignment Given)"));
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Validation Summary row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colSpan=\"2\""));
			validationSummary.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Blank row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol());
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());
			
			// Button Row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("height=\"25\""));
			btnSubmit.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.StartCol());
			btnNew.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());
			
			// Horozontal Rule row (line)
			output.Write(HTMLTools.StartRow() + HTMLTools.StartCol("colspan=\"6\"") + HTMLTools.HorozontalRule() + HTMLTools.EndCol() + HTMLTools.EndRow());

			// User maintenance section: Title row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colspan=\"2\""));
			lblUserTitle.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Users CheckBoxList
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("colSpan=\"2\""));
			cblUsers.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Submit Student Changes Button Row
			output.Write(HTMLTools.StartRow());
			output.Write(HTMLTools.StartCol("height=\"25\" colSpan=\"2\""));
			btnUsers.RenderControl(output);
			output.Write(HTMLTools.EndCol());
			output.Write(HTMLTools.EndRow());

			// Horozontal Rule row (line)
			output.Write(HTMLTools.StartRow() + HTMLTools.StartCol("colspan=\"6\"") + HTMLTools.HorozontalRule() + HTMLTools.EndCol() + HTMLTools.EndRow());

			

			// END TABLE
			output.Write(HTMLTools.EndTable());

			output.Write(HTMLTools.Comment("-=-=-= End PREP Project Administration =-=-=-"));
			
		}

		/// <summary>
		/// Submit button for new PREP Assigmnent creation, or to update 
		/// an existing PREP assignment.
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnSubmit_Click(object sender, EventArgs e)
		{
			if(btnSubmit.Text == "Create Assignment")
			{
				CreateAssignment();
				BindRepeater();
			}
			else if(btnSubmit.Text == "Update Assignment")
			{
				UpdateAssignment((string)ViewState["ASSIGNMENT_NAME"]);
			}

		}

		/// <summary>
		/// User collection update button
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void btnUsers_Click(object sender, EventArgs e)
		{
			// Check to see if the Role for this assignment exists. If it doesn't, create it.
			CreateOrUpdateRole((string)ViewState["ASSIGNMENT_NAME"]);

			// Add or delete members from the role, based on assignment name plus the contents of
			// CheckBoxList cblUsers.
			AddDeleteRoleMembers((string)ViewState["ASSIGNMENT_NAME"]);

			//Rebind the CheckBoxList
			LoadCheckBoxListFromRole((string)ViewState["ASSIGNMENT_NAME"]);
		}

		/// <summary>
		/// Item Command for the Assignment Repeater.
		/// Fires when any command button is hit.
		/// This Repeater has 2 link buttons, one for Edit, and one for Delete
		/// </summary>
		/// <param name="source"></param>
		/// <param name="e"></param>
		private void rptAssignments_ItemCommand(object source, RepeaterCommandEventArgs e)
		{
			// CommandArgument was loaded in the ItemBound Event.
			string assignmentName = e.CommandArgument.ToString();
			
			// Any button in the repeater will fire this event. Have to check which one was clicked
			// in order to know which action to take.
			switch (e.CommandName)
			{
				case "Edit":
				{
					LoadForm(assignmentName);
					ViewState.Add("ASSIGNMENT_NAME", assignmentName);
					btnSubmit.Text = "Update Assignment";
					lblTitle.Text = HTMLTools.Literal("<b>Edit PREP Assignment</b>");
					break;
				}
				case "Delete":
				{
					int numDeleted = 0;
					try
					{
						numDeleted = PREPAssignment.Delete(PREPWeb, assignmentName);
						lblMessage.Visible = true;
						lblMessage.Text = "<b>" + assignmentName + " has been deleted. There were " + numDeleted.ToString() + " Events associated with this assignment.</b>";
					}
					catch (Exception ex)
					{
						lblMessage.Visible = true;
						lblMessage.Text = "<b>" + assignmentName + " could not be deleted. <br>" + ex.Message + " </b>";
					}

					BindRepeater();
					BindCheckBoxList();
					break;
				}
				default:
				{
					break;
				}

			}

		}

		private void ClearForm()
		{
			txtAssignment.Text = "";
			txtDescription.Text = "";
			txtAssignmentDate.Text = "";
			txtIdeasDueDate.Text = "";
			txtReviewsDueDate.Text = "";

			BindCheckBoxList();

			btnSubmit.Text = "Create Assignment";
			btnSubmit.Enabled = true;

		}

		private void BindRepeater()
		{
			//Rewrite the prepAssignments Arraylist, and rebind the repeater
			prepAssignments = PREPAssignment.PREPAssignment_Collection(PREPWeb);
			rptAssignments.DataSource = prepAssignments;
			rptAssignments.DataBind();

		}

		private void BindCheckBoxList()
		{
			cblUsers.DataSource = PREPWeb.Users; // all users on the site
			cblUsers.DataTextField = "Name";
			cblUsers.DataValueField = "LoginName";
			cblUsers.DataBind();
		}

		private void LoadCheckBoxListFromRole(string assignmentName)
		{
			// load the users from the persisted SPRole object
			foreach(ListItem item in cblUsers.Items)
			{
				item.Selected = false;
				SPUser user = PREPWeb.Users[item.Value];
				foreach(SPRole role in user.Roles)
				{
					if(role.Name == assignmentName)
					{
						item.Selected = true;
					}
				}
			}
		}
		
		/// <summary>
		/// The CheckBoxList has no items on postback until it is databound.
		/// At that point, it loses any information about what boxes were checked by the user.
		/// This routine retrieves the checked state of the checkboxes from the
		/// Request object, and resets the items on the control.
		/// </summary>
		private void LoadCheckBoxListOnPostBack()
		{
			cblUsers.DataSource = PREPWeb.Users; // all users on the site
			cblUsers.DataTextField = "Name";
			cblUsers.DataValueField = "LoginName";
			cblUsers.DataBind();
			
			for(int i=0; i<cblUsers.Items.Count; i++)
			{
				string checkBoxListItemName = cblUsers.ID + ":" + i.ToString();
				cblUsers.Items[i].Selected = false;
				try
				{
					// only CheckBoxList items that are checked actually exist in the Request object upon postback .
					// Therefore, if the key we are looking for exists, it was selected.
					// The key will look something like this: "cblUsersControl:1".
					if(Context.Request.Form[checkBoxListItemName] != null)
					{
						cblUsers.Items[i].Selected = true;
					}
				}
				catch {}
			}
		}



		/// <summary>
		/// Loads values to be edited into the form fields.
		/// Assumes that there is a prepAssignments ArrayList already loaded
		/// with PREP assignments.
		/// </summary>
		/// <param name="assignmentName"></param>
		private void LoadForm(string assignmentName)
		{
			ViewState.Add("ASSIGNMENT_NAME", assignmentName);
			
			foreach(PREPAssignment assignment in prepAssignments)
			{
				if(assignment.AssignmentName == assignmentName)
				{
					// load the form fields
					txtAssignment.Text = assignment.AssignmentName;
					txtDescription.Text = assignment.Description;
					txtAssignmentDate.Text = assignment.AssignedDate.ToShortDateString();
					txtIdeasDueDate.Text = assignment.IdeasDueDate.ToShortDateString();
					TimeSpan ts = new TimeSpan();
					ts = assignment.IdeasDueDate - assignment.AssignedDate;
					txtIdeasDueDate.Text = ts.Days.ToString();
					ts = assignment.ReviewsDueDate - assignment.AssignedDate;
					txtReviewsDueDate.Text = ts.Days.ToString();

					// create the user CheckBoxList and load it from the role for this assignment
					BindCheckBoxList();
					LoadCheckBoxListFromRole(assignment.AssignmentName);

					//if this assignment has existing data (tasks or documents), do not allow update.
					if(assignment.HasData)
					{
						btnSubmit.Enabled = false;
						lblMessage.Visible = true;
						lblMessage.Text = "<b>This assignment has existing tasks or documents. Update is disabled.</b>";
					}
					else
					{
						btnSubmit.Enabled = true;
						lblMessage.Visible = false;

					}
					break;
				}
			}
		}

		private void btnNew_Click(object sender, EventArgs e)
		{
			ClearForm();
			lblTitle.Text = HTMLTools.Literal("<b>New PREP Assignment Creation</b>");
		}

		/// <summary>
		/// Routine which creates a new PREP Assignment from the data in the text boxes.
		/// </summary>
		private void CreateAssignment()
		{
			try
			{
				SPWeb PREPWeb = SPControl.GetContextWeb(Context);
				SPList eventList = PREPWeb.Lists["Events"];

				SPListItemCollection items= eventList.Items;

				PREPAssignment assignment = new PREPAssignment();

				assignment.AssignmentName = txtAssignment.Text;
				assignment.Description = txtDescription.Text;
				assignment.AssignedDate = DateTime.Parse(txtAssignmentDate.Text);
				assignment.IdeasDueDate = assignment.AssignedDate.AddDays(int.Parse(txtIdeasDueDate.Text));
				assignment.ReviewsDueDate = assignment.AssignedDate.AddDays(int.Parse(txtReviewsDueDate.Text));

				int numEvents = PREPAssignment.Create(PREPWeb, assignment);

				// Create a role for this assignment and add users to it if any were checked.
				CreateOrUpdateRole(txtAssignment.Text);
				BindCheckBoxList();
				LoadCheckBoxListOnPostBack();
				AddDeleteRoleMembers(assignment.AssignmentName);

				lblMessage.Visible = true;
				lblMessage.Text += "PREP Assignment " + txtAssignment.Text + " added successfully. " + numEvents.ToString() + " Events were created.";

				LoadForm(txtAssignment.Text);
			}
			catch(Exception ex)
			{
				lblMessage.Visible = true;
				lblMessage.Text += "Cannot create a new PREP Assignment in Web Part PREPProjectAdmin. " + ex.Message;
				PREPWorkflow.Utilities.LogError("Cannot create a new PREP Assignment in Web Part PREPProjectAdmin", ex);
			}

		}

		/// <summary>
		/// Routine which will update an existing PREP Assignment.
		/// Updates are not allowed on assignments that have data.
		/// Strictly speaking, updating is not possible because DateTime objects
		/// are immutable (as in the "Begin" - or date - fields on the Events).
		/// Therefore, Assignment Events are deleted and recreated.
		/// </summary>
		private void UpdateAssignment(string assignmentName)
		{
			foreach(PREPAssignment assignment in prepAssignments)
			{
				// find the assignment to update
				if(assignment.AssignmentName == assignmentName)
				{
					//Allow update only if there is no existing data in the task list
					//or doc library for the project.
					if(!assignment.HasData)
					{
						btnSubmit.Enabled = true;

						//Delete the existing assignment
						int numEvents = PREPAssignment.Delete(PREPWeb, assignmentName);
						BindRepeater();

						lblMessage.Visible = true;
						lblMessage.Text = "<b>Assignment " + assignment.AssignmentName + " deleted. " + numEvents.ToString() + " Events were affected.</b><br><br>";
						
						// Recreate the assignment using the values on the web form
						CreateAssignment();
						BindRepeater();
					
					}
					else
					{
						btnSubmit.Enabled = false;
						lblMessage.Visible = true;
						lblMessage.Text = "<b>Assignment " + assignment.AssignmentName + " has data - Update disabled.</b>";
					}

				} // if(assignment.AssignmentName == assignmentName)

			} // foreach(PREPAssignment assignment in prepAssignments)

		} // UpdateAssignment()


		/// <summary>
		/// Create a role if it does not already exist.
		/// </summary>
		/// <param name="roleName"></param>
		private void CreateOrUpdateRole(string roleName)
		{
			try
			{
				string[] names = new string[1];
				names[0] = roleName;
				SPRoleCollection siteRoles = PREPWeb.Roles;
				// try to see if this role already exists in the collection
				SPRoleCollection thisRole = siteRoles.GetCollection(names);
				// if the role exists, the count will be 1. If not, it will be 0.
				if(thisRole.Count == 0)
				{
					siteRoles.Add(roleName, txtDescription.Text, SPRights.AddListItems | SPRights.EditListItems | SPRights.DeleteListItems | SPRights.ViewListItems | SPRights.ViewPages | SPRights.ManagePersonalViews);
				}
			}
			catch (Exception ex)
			{
				Utilities.LogError("Cannot Create Role " + roleName + " in CreateOrUpdateRole.", ex);
			}
		} //CreateOrUpdateRole

		/// <summary>
		/// Add members to a role who are checked in a CheckBoxList, and delete members from the role who are not checked in the CheckBoxList
		/// </summary>
		/// <param name="roleName">The role name, which is equal to the assignment name.</param>
		private void AddDeleteRoleMembers(string roleName)
		{
			try
			{
				SPRole role = PREPWeb.Roles[roleName];
				SPUser user;
			
				// 1. Add users from the CheckBoxList to the role
				foreach(ListItem item in cblUsers.Items)
				{
					if(item.Selected == true)
					{
						user = PREPWeb.AllUsers[item.Value];
						role.AddUser(user); // this apparently won't do anything if the user already exists.
						// Each new user added to the role is assigned a task to write an ideas document
						CreateIdeasTask(ref role, ref user);
					}
				} // Add Loop
				CreateAndUpdateReviewTasks(ref role);

				// 2. Delete users from the role who are not checked in the repeater
				SPUserCollection users = role.Users;
				foreach(SPUser roleUser in users)
				{
					foreach(ListItem item in cblUsers.Items)
					{
						// if role user == CheckBoxList user. The value of the CheckBoxList item is the LoginName
						if(roleUser.LoginName == item.Value)
						{
							if(!item.Selected)
							{
								string roleUserLoginName = Utilities.GetLoginName(roleUser.LoginName);

								// 1. remove all tasks assigned to this user.
								// Any documents and folders will be left as-is.
								PREPTask.DeleteTasksAssignedToUser(PREPWeb, roleName, roleUserLoginName);

								// 2. remove all tasks in which others are assigned to review this user's ideas
								PREPTask.DeleteTasksRelatedToUser(PREPWeb, roleName, roleUserLoginName);
								
								// 3. Remove the user from the role.
								SPUser itemUser = PREPWeb.AllUsers[item.Value];
								role.RemoveUser(itemUser);
							}
						}
					}
				} // Delete loop
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in AddDeleteRoleMembers.", ex);
			}

		} // AddDeleteRoleMembers

		/// <summary>
		/// Create an "ideas" tasks for a given user for a given assignment.
		/// Each user will have one ideas task (the obligation to upload an ideas document).
		/// This method will NOT create a task if one already exists
		/// </summary>
		/// <param name="role">Custom role for the assignment. It contains the list of users on the assignment.</param>
		/// <param name="user">The user to create an Ideas Task for.</param>
		private void CreateIdeasTask(ref SPRole role, ref SPUser user)
		{
			string assignmentName = role.Name; // The assignment name equals the role name
			string currentUserLoginName = Utilities.GetLoginName(user.LoginName);
			
			// 1. IDEAS TASK.
			// There should be one "ideas" task per user per assignment. 
			// This task is each user's obligation to create and upload an ideas document.
			// The ideas task is defined as one where the task's document Author == the task's assigned-to user.
			// The "document author" is the author of the ideas document this task pertains to.
			try
			{
				
				// Get the PREP Task list
				SPList PREPList = PREPWeb.Lists[prepTaskListName];

				// First, see if this task already exists. If it does, do not recreate it.
				
				// Populate an ArrayList of PREP Task objects for the current assignment
				ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPList, assignmentName);
				
				// If the task does not exist, create it.
				PREPTask ideasTask = PREPTask.GetIdeasTask(prepTasks, assignmentName, currentUserLoginName);
				if(!ideasTask.Exists)
				{
					PREPTask.Create(PREPWeb, PREPList, assignmentName, "", currentUserLoginName, user);
					// If an ideas document already exists, set the task status to completed.
					// This could happen if a user is deleted from a project, then added back later.
					// Deleting a user from a project will not delete that user's documents.
					PREPDocument ideasDoc = PREPDocument.GetIdeasDoc(PREPWeb, assignmentName, currentUserLoginName);
					if(ideasDoc.Exists)
					{
						// repopulate prepTasks to pick up the new ideas task
						prepTasks = PREPTask.PREPTask_Collection(PREPList, assignmentName);
						// get the new ideas task
						PREPTask newIdeasTask = PREPTask.GetIdeasTask(prepTasks, assignmentName, currentUserLoginName);
						// update the new ideas task
						newIdeasTask.SPListItemObject["Status"] = PREPFields.Completed;
						newIdeasTask.SPListItemObject.Update();
					}
				}

			}
			catch (Exception ex)
			{
				Utilities.LogError("Cannot create PREP task in method CreateIdeasTask(). currentUserLoginName="+currentUserLoginName+", assignmentName="+assignmentName, ex);
			}

		} // CreateIdeasTask()

		private void CreateAndUpdateReviewTasks(ref SPRole role)
		{
			try
			{
				string assignmentName = role.Name; // The assignment name equals the role name

				// Get the PREP Task list
				SPList PREPList = PREPWeb.Lists[prepTaskListName];

				// Populate an ArrayList of PREP Task objects
				ArrayList prepTasks = PREPTask.PREPTask_Collection(PREPList, assignmentName);

				// Loop through the users in the role, to create a review task for each.
				SPUserCollection users = role.Users;
				// The "ideas" user is analogous to the column going down in the PREP Matrix
				// It means the user whose ideas the task pertains to, i.e. 
				//   Ideas task means the ideasUser is tasked with creating an ideas document
				//   Review task means the assignmentUser is tasked with reviewing the ideasUsers ideas document
				foreach(SPUser ideasUser in users)
				{
					string ideasUserLoginName = Utilities.GetLoginName(ideasUser.LoginName);

					// The "assignment" user is analogous to the row going across in the PREP Matrix.
					// It means the user who is assigned a task to review the ideasUser's ideas document.
					foreach(SPUser assignmentUser in users)
					{
						string assignmentUserLoginName = Utilities.GetLoginName(assignmentUser.LoginName);
						
						// Tasks where ideasUserLoginName != assignmentUserLoginName are Review tasks. if they are equal, it is ideasUserLoginName's ideas task.
						if(ideasUserLoginName != assignmentUserLoginName)
						{
							// Create a review task if one does not already exist.
							// assignmentUserLoginName needs to review ideasUserLoginName's ideas.
							PREPTask reviewTask = PREPTask.GetReviewTask(prepTasks, assignmentName, ideasUserLoginName, assignmentUserLoginName);
							if(!reviewTask.Exists)
							{
								// Only create a review task if there is a document to review, i.e. the ideasUser has uploaded an ideas document.
								// If there is no ideas document, the tasks will be created by the event handler when the ideas author uploads one.
								PREPDocument ideaUsersPREPDoc = PREPDocument.GetIdeasDoc(PREPWeb, assignmentName, ideasUserLoginName);
								if(ideaUsersPREPDoc.Exists)
								{
									PREPTask.Create(PREPWeb, PREPList, assignmentName, ideaUsersPREPDoc.FileName, ideasUserLoginName, assignmentUser);
									
									// Now see if there is an existing review document for this task. This could happen if an ideas doc was deleted and reuploaded.
									// If there is, we need to update the status to Completed.
									PREPDocument reviewDoc = PREPDocument.GetReviewDoc(PREPWeb, assignmentName, ideasUserLoginName, assignmentUserLoginName);
									if(reviewDoc.Exists)
									{
										// repopulate prepTasks to pick up the new review task
										prepTasks = PREPTask.PREPTask_Collection(PREPList, assignmentName);
										// get the new review task
										PREPTask newReviewTask = PREPTask.GetReviewTask(prepTasks, assignmentName, ideasUserLoginName, assignmentUserLoginName);
										// update the new Review task
										newReviewTask.SPListItemObject["Status"] = PREPFields.Completed;
										newReviewTask.SPListItemObject.Update();
									}
								}
							}

						} // if(assignmentUserLoginName != currentUserLoginName)
						
					} // foreach(SPUser assignmentUser in users)

				} // foreach(SPUser ideasUser in users)
			}
			catch (Exception ex)
			{
				Utilities.LogError("Cannot create PREP task in method CreateAndUpdateReviewTasks().", ex);
			}

		} //CreateAndUpdateReviewTasks()

				
	} // class PREPProjectAdmin


	/// <summary>
	/// Class to handle the ItemTemplate for the Assignments Repeater. 
	/// </summary>
	public class RepeaterColumn : ITemplate
	{
		#region ITemplate Members

		/// <summary>
		/// Required method of the ITemplate interface.
		/// </summary>
		/// <param name="container"></param>
		public void InstantiateIn(Control container)
		{
			Literal ltl1 = new Literal();
			Literal ltl2 = new Literal();
			Literal ltl3 = new Literal();
			Literal ltl4 = new Literal();
			Literal ltl5 = new Literal();

			Label lblAssignment = new Label();
			Label lblDates = new Label();
			LinkButton editLink = new LinkButton();
			LinkButton delLink = new LinkButton();
			lblAssignment.DataBinding += new EventHandler(lblAssignment_DataBinding);
			lblDates.DataBinding +=new EventHandler(lblDates_DataBinding);
			editLink.DataBinding +=new EventHandler(editLink_DataBinding);
			delLink.DataBinding += new EventHandler(delLink_DataBinding);

			//ltl1.Text = HTMLTools.StartRow() + HTMLTools.StartCol("width=\"50%\"");
			ltl1.Text = HTMLTools.StartRow();
			container.Controls.Add(ltl1);
			container.Controls.Add(lblAssignment);
			container.Controls.Add(lblDates);
			
			ltl2.Text = HTMLTools.StartCol();
			container.Controls.Add(ltl2);
			container.Controls.Add(editLink);
			ltl3.Text = HTMLTools.EndCol();
			container.Controls.Add(ltl3);
			
			ltl4.Text = HTMLTools.StartCol();
			container.Controls.Add(ltl4);
			container.Controls.Add(delLink);
			ltl5.Text = HTMLTools.EndCol() + HTMLTools.EndRow();
			container.Controls.Add(ltl5);
		}

		#endregion

		private void lblAssignment_DataBinding(object sender, EventArgs e)
		{
			Label lbl = (Label)sender;
			RepeaterItem container = (RepeaterItem)lbl.NamingContainer;
			lbl.Text = HTMLTools.StartCol() + Convert.ToString(DataBinder.Eval(((RepeaterItem)container).DataItem, "AssignmentName")) + HTMLTools.EndCol();
		}

		private void editLink_DataBinding(object sender, EventArgs e)
		{
			LinkButton lb = (LinkButton)sender;
			RepeaterItem container = (RepeaterItem)lb.NamingContainer;
			lb.Text = "Edit";
			lb.CommandName = "Edit";
			lb.CausesValidation = false;
			lb.CommandArgument  = Convert.ToString(DataBinder.Eval(((RepeaterItem)container).DataItem, "AssignmentName"));
		}

		private void delLink_DataBinding(object sender, EventArgs e)
		{
			LinkButton lb = (LinkButton)sender;
			RepeaterItem container = (RepeaterItem)lb.NamingContainer;
			string assignmentName = Convert.ToString(DataBinder.Eval(((RepeaterItem)container).DataItem, "AssignmentName"));
			lb.Text = "Delete";
			lb.CommandName = "Delete";
			lb.CausesValidation = false;
			lb.CommandArgument  = assignmentName;

			//Add a JavaScript "Are you sure" button.
			lb.Attributes.Add("onclick", "javascript:if(confirm('Are you sure you want to delete assignment " + assignmentName + "?')== false) return false;");
		}

		private void lblDates_DataBinding(object sender, EventArgs e)
		{
			Label lbl = (Label)sender;
			RepeaterItem container = (RepeaterItem)lbl.NamingContainer;
			lbl.Text = HTMLTools.StartCol() + DateTime.Parse(Convert.ToString(DataBinder.Eval(((RepeaterItem)container).DataItem, "AssignedDate"))).ToShortDateString() + HTMLTools.EndCol();
			lbl.Text += HTMLTools.StartCol() + DateTime.Parse(Convert.ToString(DataBinder.Eval(((RepeaterItem)container).DataItem, "IdeasDueDate"))).ToShortDateString() + HTMLTools.EndCol();
			lbl.Text += HTMLTools.StartCol() + DateTime.Parse(Convert.ToString(DataBinder.Eval(((RepeaterItem)container).DataItem, "ReviewsDueDate"))).ToShortDateString() + HTMLTools.EndCol();
		}
	}

	/// <summary>
	/// Class to handle the Header Templage for the Assignments Repeater
	/// </summary>
	public class RepeaterHeader : ITemplate
	{
		#region ITemplate Members

		/// <summary>
		/// Required method of the ITemplate interface.
		/// </summary>
		/// <param name="container"></param>
		public void InstantiateIn(Control container)
		{
			// TODO:  Add RepeaterHeader.InstantiateIn implementation
			Literal ltl = new Literal();
			ltl.Text = HTMLTools.StartTable() + HTMLTools.StartRow() + HTMLTools.StartCol("colspan=\"6\"") + HTMLTools.Literal("<b>PREP Assignment List</b>") + HTMLTools.EndCol() + HTMLTools.EndRow();
			ltl.Text += HTMLTools.StartRow() + HTMLTools.StartCol() + "<b>Assignment Name</b>" + HTMLTools.EndCol();
			ltl.Text += HTMLTools.StartCol() + "<b>Assigned</b>" + HTMLTools.EndCol();
			ltl.Text += HTMLTools.StartCol() + "<b>Ideas Due</b>" + HTMLTools.EndCol();
			ltl.Text += HTMLTools.StartCol() + "<b>Reviews Due</b>" + HTMLTools.EndCol();
			ltl.Text += HTMLTools.StartCol() + "<b>Edit</b>" + HTMLTools.EndCol();
			ltl.Text += HTMLTools.StartCol() + "<b>Delete</b>" + HTMLTools.EndCol() + HTMLTools.EndRow();
			container.Controls.Add(ltl);
		}

		#endregion

	}

	/// <summary>
	/// Class to handle the Footer Template of the Assignments Repeater.
	/// </summary>
	public class RepeaterFooter : ITemplate
	{
		#region ITemplate Members

		/// <summary>
		/// Required method of the ITemplate interface.
		/// </summary>
		/// <param name="container"></param>
		public void InstantiateIn(Control container)
		{
			Literal ftr = new Literal();
			ftr.Text = HTMLTools.StartRow() + HTMLTools.StartCol("colspan=\"6\"") + HTMLTools.HorozontalRule() + HTMLTools.EndCol() + HTMLTools.EndRow() + HTMLTools.EndTable();
			container.Controls.Add(ftr);
		}

		#endregion

	}




}
