using System;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml.Serialization;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebPartPages;

using Microsoft.SharePoint.WebControls;

namespace PREPWebPartLibrary20
{
	/// <summary>
	/// The User Alert Administration Web Part. This Web Part allows an administrator to easily set up user
	/// alerts for everyone on a SharePoint Site. The administrator may choose to set up an 
	/// alert for any list on the site. The alerts can be set up for everyone or just selected 
	/// users. All of the normal alert options (immediate, daily summary, etc.) are available. 
	/// Web Part by Jan Tielens: 
	/// http://weblogs.asp.net/jan/archive/2004/03/27/97335.aspx
	/// </summary>
	[ToolboxData("<{0}:UserAlerts runat=server></{0}:UserAlerts>"),
		XmlRoot(Namespace="PREPWebPartLibrary20")]
	public class UserAlerts : Microsoft.SharePoint.WebPartPages.WebPart
	{	
		System.Web.UI.WebControls.
		DropDownList listList;
		ListBox alertList;
		DropDownList userList;
		LinkButton removeAlert;
		LinkButton addAlert;
		LinkButton removeAllAlerts;
		LinkButton addAllAlerts;
		Label status;

		RadioButton alertTypeAll;
		RadioButton alertTypeAdded;
		RadioButton alertTypeChanged;
		RadioButton alertTypeDeleted;

		RadioButton alertFrequencyImmediately;
		RadioButton alertFrequencyDaily;
		RadioButton alertFrequencyWeekly;

		/// <summary>
		/// Render this Web Part to the output parameter specified.
		/// </summary>
		/// <param name="output"> The HTML writer to write out to </param>
		protected override void RenderWebPart(HtmlTextWriter output)
		{
			SPWeb web = SPControl.GetContextWeb(this.Context);
			if(!web.CurrentUser.IsSiteAdmin)
			{
				output.Write("<b>To use this webpart you need to have Site Administrator rights.</b>");
			}
			
			output.WriteLine("<table width='100%'>");
			output.WriteLine("<tr>");
			output.WriteLine("<td>");
			listList.RenderControl(output);
			output.WriteLine("</td>");
			output.WriteLine("</tr>");

			output.WriteLine("<tr>");
			output.WriteLine("<td>");
			alertList.RenderControl(output);
			output.WriteLine("<br>");
			removeAlert.RenderControl(output);
			output.WriteLine("<br>");
			removeAllAlerts.RenderControl(output);
			output.WriteLine("</td>");
			output.WriteLine("</tr>");
			
			output.WriteLine("<tr>");
			output.WriteLine("<td>");
			output.WriteLine("&nbsp;");
			output.WriteLine("</td>");
			output.WriteLine("</tr>");
			
			output.WriteLine("<tr>");
			output.WriteLine("<td>");
			userList.RenderControl(output);
			output.WriteLine("<br>");
			addAlert.RenderControl(output);
			output.Write("<br>");
			addAllAlerts.RenderControl(output);
			output.WriteLine("</td>");
			output.WriteLine("</tr>");

			output.WriteLine("<tr>");
			output.WriteLine("<td>");
			output.WriteLine("<b>New alert type</b><br>");
			
			output.WriteLine("<table width='100%'>");
			output.WriteLine("<tr valign=top>");
			output.WriteLine("<td>");
			output.WriteLine("Change type:<br>");
			alertTypeAll.RenderControl(output);
			output.WriteLine("<br>");
			alertTypeAdded.RenderControl(output);
			output.WriteLine("<br>");
			alertTypeChanged.RenderControl(output);
			output.WriteLine("<br>");
			alertTypeDeleted.RenderControl(output);
			output.WriteLine("</td>");
			output.WriteLine("<td>");
			output.WriteLine("Alert frequency:<br>");
			alertFrequencyImmediately.RenderControl(output);
			output.WriteLine("<br>");
			alertFrequencyDaily.RenderControl(output);
			output.WriteLine("<br>");
			alertFrequencyWeekly.RenderControl(output);
			output.WriteLine("</td>");
			output.WriteLine("</tr>");
			output.WriteLine("</table>");
			output.WriteLine("</td>");
			output.WriteLine("</tr>");

			output.WriteLine("<tr>");
			output.WriteLine("<td>");
			status.RenderControl(output);
			output.WriteLine("</td>");
			output.WriteLine("</tr>");
			
			output.WriteLine("</table>");
		}

		/// <summary>
		/// Method which creates the ASP.NET controls on a Web Part.
		/// </summary>
		protected override void CreateChildControls()
		{
			base.CreateChildControls ();			
			SPWeb web = SPControl.GetContextWeb(this.Context);
			
			//listList
			listList = new DropDownList();
			listList.EnableViewState = false;
			listList.Width = new Unit("100%");
			listList.AutoPostBack = true;
			foreach(SPList lst in web.Lists)
			{
				ListItem li = new ListItem(lst.Title, lst.ID.ToString());
				listList.Items.Add(li);
			}
			listList.SelectedIndexChanged += new EventHandler(list_SelectedIndexChanged);
			this.Controls.Add(listList);

			//alertList
			alertList = new ListBox();
			alertList.EnableViewState = false;
			alertList.Width = new Unit("100%");
			alertList.Height = 200;
			this.Controls.Add(alertList);

			//userList
			userList = new DropDownList();
			userList.EnableViewState = false;
			userList.Width = new Unit("100%");
			foreach(SPUser user in web.Users) // web.SiteUsers)
			{
				userList.Items.Add(new ListItem(user.Name, user.LoginName));
			}
			this.Controls.Add(userList);

			//linkButtons
			addAlert = new LinkButton();
			addAlert.Text = "Add alert for selected user";
			addAlert.Click +=new EventHandler(addAlert_Click);
			this.Controls.Add(addAlert);
			removeAlert = new LinkButton();
			removeAlert.Text = "Remove selected alert";
			removeAlert.Click += new EventHandler(removeAlert_Click);
			this.Controls.Add(removeAlert);
			removeAllAlerts = new LinkButton();
			removeAllAlerts.Text = "Remove all alerts";
			removeAllAlerts.Click += new EventHandler(removeAllAlerts_Click);
			this.Controls.Add(removeAllAlerts);
			addAllAlerts = new LinkButton();
			addAllAlerts.Text = "Add alert for all users";
			addAllAlerts.Click += new EventHandler(addAllAlerts_Click);
			this.Controls.Add(addAllAlerts);

			//status
			status = new Label();
			status.EnableViewState = false;
			this.Controls.Add(status);

			//alert radio buttons
			alertTypeAll = new RadioButton();
			alertTypeAll.Text = "All changes";
			alertTypeAll.AutoPostBack = false;
			alertTypeAll.Checked = true;
			this.Controls.Add(alertTypeAll);
			alertTypeAdded = new RadioButton();
			alertTypeAdded.Text = "Added items";
			alertTypeAdded.AutoPostBack = false;
			this.Controls.Add(alertTypeAdded);
			alertTypeChanged = new RadioButton();
			alertTypeChanged.Text = "Changed items";
			alertTypeChanged.AutoPostBack = false;
			this.Controls.Add(alertTypeChanged);
			alertTypeDeleted = new RadioButton();
			alertTypeDeleted.Text = "Deleted items";
			alertTypeDeleted.AutoPostBack = false;
			this.Controls.Add(alertTypeDeleted);
			alertTypeAll.GroupName = "AlertType";
			alertTypeAdded.GroupName = "AlertType";
			alertTypeChanged.GroupName = "AlertType";
			alertTypeDeleted.GroupName = "AlertType";
			alertFrequencyImmediately = new RadioButton();
			alertFrequencyImmediately.Text = "Immediately";
			alertFrequencyImmediately.AutoPostBack = false;
			alertFrequencyImmediately.Checked = true;
			this.Controls.Add(alertFrequencyImmediately);
			alertFrequencyDaily = new RadioButton();
			alertFrequencyDaily.Text = "Daily";
			alertFrequencyDaily.AutoPostBack = false;
			this.Controls.Add(alertFrequencyDaily);
			alertFrequencyWeekly = new RadioButton();
			alertFrequencyWeekly.Text = "Weekly";
			alertFrequencyWeekly.AutoPostBack = false;
			this.Controls.Add(alertFrequencyWeekly);
			alertFrequencyImmediately.GroupName = "AlertFrequency";
			alertFrequencyDaily.GroupName = "AlertFrequency";
			alertFrequencyWeekly.GroupName = "AlertFrequency";


			DisplaySelectedList();
		}

		private void list_SelectedIndexChanged(object sender, EventArgs e)
		{
			DisplaySelectedList();
		}

		
		private void removeAlert_Click(object sender, EventArgs e)
		{
			if(alertList.SelectedItem != null)
			{
				try
				{
					SPWeb web = SPControl.GetContextWeb(this.Context);
					web.Alerts.Delete(new System.Guid(alertList.SelectedItem.Value));
					alertList.Items.Remove(alertList.SelectedItem);
				}
				catch(Exception ex)
				{
					status.Text = string.Format("<b>{0}</b>", ex.ToString());
				}
			}
			else
			{
				status.Text = string.Format("<b>{0}</b>", "No alert selected!");
			}
		}
		
		private void addAlert_Click(object sender, EventArgs e)
		{
			try
			{
				SPWeb web = SPControl.GetContextWeb(this.Context);

				SPList selectedList = web.Lists[new System.Guid(listList.SelectedItem.Value)];
				SPUser selectedUser = web.Users[userList.SelectedItem.Value]; //web.SiteUsers[userList.SelectedItem.Value];

				System.Guid newAlertGuid = selectedUser.Alerts.Add(selectedList, GetSelectedChangeType(), GetSelectedFrequency());
				AddAlertToList(selectedUser.Alerts[newAlertGuid]);
			}
			catch(Exception ex)
			{
				status.Text = string.Format("<b>{0}</b>", ex.ToString());
			}
		}

		private SPAlertFrequency GetSelectedFrequency()
		{
			SPAlertFrequency frequencyType = SPAlertFrequency.Immediate;

			if(alertFrequencyImmediately.Checked)
				frequencyType = SPAlertFrequency.Immediate;
			if(alertFrequencyDaily.Checked)
				frequencyType = SPAlertFrequency.Daily;
			if(alertFrequencyWeekly.Checked)
				frequencyType = SPAlertFrequency.Weekly;

			return frequencyType;
		}

		private SPEventType GetSelectedChangeType()
		{
			SPEventType changeType = SPEventType.All;
			if(alertTypeAll.Checked)
				changeType = SPEventType.All;
			else if(alertTypeAdded.Checked)
				changeType = SPEventType.Add;
			else if(alertTypeChanged.Checked)
				changeType = SPEventType.Modify;
			else if(alertTypeDeleted.Checked)
				changeType = SPEventType.Delete;

			return changeType;
		}

		private void DisplaySelectedList()
		{
			alertList.Items.Clear();

			SPWeb web = SPControl.GetContextWeb(this.Context);
			SPList selectedList = web.Lists[new System.Guid(listList.SelectedItem.Value)];

			foreach(SPAlert alert in web.Alerts)
			{
				if(alert.List.ID == selectedList.ID)
				{
					AddAlertToList(alert);
				}
			}
		}

		private void removeAllAlerts_Click(object sender, EventArgs e)
		{
			try
			{
				SPWeb web = SPControl.GetContextWeb(this.Context);
				SPList selectedList = web.Lists[new System.Guid(listList.SelectedItem.Value)];
				foreach(ListItem li in alertList.Items)
				{
						web.Alerts.Delete(new System.Guid(li.Value));
				}
				alertList.Items.Clear();
			}
			catch(Exception ex)
			{
				status.Text = string.Format("<b>{0}</b>", ex.ToString());
			}
		}

		private void addAllAlerts_Click(object sender, EventArgs e)
		{
			SPWeb web = SPControl.GetContextWeb(this.Context);
			SPList selectedList = web.Lists[new System.Guid(listList.SelectedItem.Value)];

			foreach(SPUser user in web.Users) //web.SiteUsers)
			{
				System.Guid newAlertGuid = user.Alerts.Add(selectedList, GetSelectedChangeType(), GetSelectedFrequency());
				AddAlertToList(user.Alerts[newAlertGuid]);
			}
		}

		private void AddAlertToList(SPAlert alert)
		{
			string itemText = string.Format("{0} - {1}/{2}", alert.User.Name, alert.AlertFrequency.ToString(), alert.EventType.ToString());
			alertList.Items.Add(new ListItem(itemText, alert.ID.ToString()));
		}
	}
}
