/*
 * Copyright (c) 2006 The Massachusetts Institute of Technology. All rights reserved.
 * Please see license.txt in top level directory for full license.
 * 
 */
using System;
using System.Configuration;

namespace PREPWorkflow
{
	/// <summary>
	/// Building blocks for manual HTML construction.
	/// </summary>
	public class HTMLTools
	{
		/// <summary>
		/// Default Constructor
		/// </summary>
		public HTMLTools()
		{
		}

		/// <summary>
		/// Begins an HTML page
		/// </summary>
		/// <returns></returns>
		public static string StartPage()
		{
			return "<HTML>" + Environment.NewLine;
		}

		/// <summary>
		/// Ends an HTML page
		/// </summary>
		/// <returns></returns>
		public static string EndPage()
		{ return "</HTML>" + Environment.NewLine;}

		/// <summary>
		/// Begins a STYLE tag
		/// </summary>
		/// <returns></returns>
		public static string StartStyle()
		{
			return "<STYLE>" + Environment.NewLine;
		}

		/// <summary>
		/// Ends a STYLE tag
		/// </summary>
		/// <returns></returns>
		public static string EndStyle()
		{ return "</STYLE>" + Environment.NewLine;}
		
		/// <summary>
		/// Begins a HEAD tag
		/// </summary>
		/// <returns></returns>
		public static string StartHead()
		{
			return "<HEAD>" + Environment.NewLine;
		}

		/// <summary>
		/// Ends a HEAD tag
		/// </summary>
		/// <returns></returns>
		public static string EndHead()
		{ return "</HEAD>" + Environment.NewLine;}
		
		/// <summary>
		/// Outputs a TITLE tag
		/// </summary>
		/// <returns></returns>
		public static string Title(string titleText)
		{
			return "<TITLE>" + titleText + "</TITLE>" + Environment.NewLine;
		}
		
		
		
		/// <summary>
		/// Begins the body of a page (BODY)
		/// </summary>
		/// <returns></returns>
		public static string StartBody()
		{
			return "<BODY>" + Environment.NewLine;
		}

		/// <summary>
		/// Begins the body of a page (BODY)
		/// </summary>
		/// <param name="attrs">Attributes for the BODY tag</param>
		/// <returns></returns>
		public static string StartBody(string attrs)
		{
			if(attrs != null)
			{
				return "<BODY " + attrs + ">" + Environment.NewLine;
			}
			else
			{
				return "<BODY>" + Environment.NewLine;
			}
		}

		/// <summary>
		/// Ends the body of a page
		/// </summary>
		/// <returns></returns>
		public static string EndBody()
		{ return "</BODY>" + Environment.NewLine;}

		/// <summary>
		/// Begins a table
		/// </summary>
		/// <returns></returns>
		public static string StartTable()
		{
			return "<TABLE>" + Environment.NewLine;
		}

		/// <summary>
		/// Begins a table
		/// </summary>
		/// <param name="attrs">Attributes for the TABLE tag</param>
		/// <returns></returns>
		public static string StartTable(string attrs)
		{
			if(attrs != null)
			{
				return "<TABLE " + attrs + ">" + Environment.NewLine;
			}
			else
			{
				return "<TABLE>" + Environment.NewLine;
			}
		}

		/// <summary>
		/// Ends a Table.
		/// </summary>
		/// <returns></returns>
		public static string EndTable()
		{ return "</TABLE>" + Environment.NewLine;}
        
		/// <summary>
		/// Starts a table row, with no attributes
		/// </summary>
		/// <returns></returns>
		public static string StartRow()
		{ return "<TR>" + Environment.NewLine;}

		/// <summary>
		/// Begins a Table Row
		/// </summary>
		/// <param name="attrs">Attributes for the TR tag</param>
		/// <returns></returns>
		public static string StartRow(string attrs)
		{ 
			if(attrs != null)
			{
				return "<TR " + attrs + ">" + Environment.NewLine;
			}
			else
			{
				return "<TR>" + Environment.NewLine;
			}
		}

		/// <summary>
		/// Ends a table row
		/// </summary>
		/// <returns></returns>
		public static string EndRow()
		{ return "</TR>" + Environment.NewLine;}

		/// <summary>
		/// Starts a table column
		/// </summary>
		/// <param name="attrs">Attributes for the TD tag</param>
		/// <returns></returns>
		public static string StartCol(string attrs)
		{ 
			if(attrs != null)
			{
				return "<TD " + attrs + ">" + Environment.NewLine;
			}
			else
			{
				return "<TD>" + Environment.NewLine;
			}
		}

		/// <summary>
		/// Starts a table column, with no attributes
		/// </summary>
		/// <returns></returns>
		public static string StartCol()
		{ return "<TD>" + Environment.NewLine;}

		/// <summary>
		/// Ends a table column
		/// </summary>
		/// <returns></returns>
		public static string EndCol()
		{ return "</TD>"  + Environment.NewLine;}

		/// <summary>
		/// Creates an empty paragraph.
		/// </summary>
		/// <returns></returns>
		public static string EmptyLine()
		{ return "<P>&nbsp;</P>" + Environment.NewLine;}

		/// <summary>
		/// Emits an image tag
		/// </summary>
		/// <param name="imgSrc">File name w/o path (path is handled in the method)</param>
		/// <param name="altTag">Alt text for the image</param>
		/// <returns></returns>
		public static string Image(string imgSrc, string altTag)
		{
			return "<img border=\"0\" src=\"" + ConfigurationSettings.AppSettings["PREPWorkflow.ImagePath"] + imgSrc + "\" alt=\"" + altTag + "\" />";
		}

		/// <summary>
		/// Creates a line break (BR)
		/// </summary>
		/// <returns></returns>
		public static string Break()
		{ return "<BR/>"  + Environment.NewLine;}

		/// <summary>
		/// Creates a horozontal rule (HR)
		/// </summary>
		/// <returns></returns>
		public static string HorozontalRule()
		{ return "<HR/>" + Environment.NewLine;}

		/// <summary>
		/// Creates a horozontal rule (HR) with attributes
		/// </summary>
		/// <returns></returns>
		public static string HorozontalRule(string attrs)
		{
			if(attrs != null)
			{
				return "<HR " + attrs + ">" + Environment.NewLine;
			}
			else
			{
			  return "<HR>" + Environment.NewLine;
			}
		  }

		/// <summary>
		/// Creates a Span tag (SPAN) with attributes
		/// </summary>
		/// <returns></returns>
		public static string StartSpan(string attrs)
		{
			if(attrs != null)
			{
				return "<SPAN " + attrs + ">" + Environment.NewLine;
			}
			else
			{
				return "<SPAN>" + Environment.NewLine;
			}
		}

		/// <summary>
		/// Ends a SPAN tag
		/// </summary>
		/// <returns></returns>
		public static string EndSpan()
		{ return "</SPAN>"  + Environment.NewLine;}

		/// <summary>
		/// Creates an HTML comment
		/// </summary>
		/// <returns></returns>
		public static string Comment(string commentText)
		{
			return "<!--" + commentText + "-->" + Environment.NewLine;
		}

		/// <summary>
		/// Outputs literal text with a line break
		/// </summary>
		/// <returns></returns>
		public static string Literal(string literalText)
		{
			return literalText + Environment.NewLine;
		}

		/// <summary>
		/// Creates a FORM tag with attributes
		/// </summary>
		/// <param name="action">The target URL of the FORM post</param>
		/// <param name="method">The method: POST or GET</param>
		/// <returns></returns>
		public static string StartForm(string method, string action)
		{
			return "<FORM method=\"" + method + "\" action=\"" + action + "\">" + Environment.NewLine;
		}

		/// <summary>
		/// Ends a FORM tag
		/// </summary>
		/// <returns></returns>
		public static string EndForm()
		{ return "</FORM>"  + Environment.NewLine;}

		/// <summary>
		/// Outputs an INPUT tag with attributes.
		/// </summary>
		/// <param name="attrs">Attributes for the INPUT tag</param>
		/// <returns></returns>
		public static string Input(string attrs)
		{
			return "<INPUT " + attrs + ">" + Environment.NewLine;
		}


	}
}
