using System;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
using System.Configuration;
using System.Text;

namespace PREPWorkflow
{
	/// <summary>
	/// This handler intercepts clicks from the PREPWebPartMatrix Web Part.
	/// Depending on the context (user and document), it will download / checkout
	/// or upload /checkin a document.
	/// NOTE: THIS IS USED IN VERSION 1.0, BUT IS BEING RETIRED IN VERSION 2.0
	///        VERSION 2.0 NO LONGER HAS CHECK-IN OR CHECK-OUT.
	/// </summary>
	public class PREPCheckinCheckoutHandler : IHttpHandler
	{
		private bool reuseHandler;

		public PREPCheckinCheckoutHandler()
		{
			reuseHandler = true;
		}
		/// <summary>
		/// Process the HTTP request called by the application process
		/// Part of IHttpHandler.
		/// </summary>
		/// <param name="context"></param>
		public void ProcessRequest(HttpContext context)
		{
			string fileName = "";
			string action = "";

			try
			{
				if(context.Request["fileName"] != null)
				{
					fileName = context.Request["fileName"].ToString();
				}
				if(context.Request["prepAction"] != null)
				{
					action = context.Request["prepAction"].ToString();
				}

			
				SPWeb PREPWeb = SPControl.GetContextWeb(context);
				PREPWeb.AllowUnsafeUpdates = true;

				// The SPFolders representation of the document library has no spaces, whereas the SPList version does.
				// i.e. SPFolders representation = "PREPDocuments"; SPList representation = "PREP Documents". Hence the Replace function below.
				// This is an artifact of reconstituting a site from a template; it won't happen if you build the library manually.
				SPFolder docLibFolder = PREPWeb.Folders[ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"].Replace(" ", "")];
				SPFileCollection docLibFiles = docLibFolder.Files;
				SPFile PREPFile = docLibFiles[fileName];

				string path = PREPWeb.Url + "/" + PREPFile.Url;
				
				// Checkin or Checkout the file, if indicated
				switch(action)
				{
					case "checkout":
						// Check the file out
						PREPFile.CheckOut();
						// force a download of the file. 
						// In SharePoint, files do not live on the file system of the server... they live in SQL Server.
						context.Response.Redirect(path);
						break;
					case "checkin":
						// Here we'll redirect to the upload page. If they actually upload the file, 
						// the HandleUPdate() method in the event handler PREPEventSink will take care of checkin.
						// The only caveat is that they have to upload the correct file name or nothing will happen.
						StringBuilder sRedirect = new StringBuilder();
						sRedirect.Append(PREPWeb.Url + "/");
						sRedirect.Append(ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"].Replace(" ", ""));
						sRedirect.Append("/Forms/Upload.aspx");
						string sRedirectURL = SPEncode.HtmlEncode(sRedirect.ToString());
						context.Response.Redirect(sRedirectURL);
						break;
					case "download":
						// force a download of the file. 
						context.Response.Redirect(path);
						break;
					default:
						break;
				}

			}
			catch (Exception ex)
			{
				// The "Thread was being aborted." message happens as the result of a redirect, 
				// so for our purposes it is not an exception.
				if(ex.Message != "Thread was being aborted.")
				{
					Utilities.LogError("Error in PREPCheckinCheckoutHandler. ", ex);
				}
			}
            
		}

		/// <summary>
		/// Property to indicate class reuse state
		/// Part of IHttpHandler
		/// </summary>
		public bool IsReusable
		{
			get
			{
				return reuseHandler;
			}
		}
	}
}
