using System;
using System.Web;
using System.Web.Security;
using System.Web.SessionState;
using System.Web.UI;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
using System.Configuration;
using System.Text;

namespace PREPWorkflow
{
	/// <summary>
	/// Popup page from the PREP Matrix. This page handles document
	/// download and checkout.
	/// </summary>
	public class PREPPopup : IHttpHandler
	{
		private bool reuseHandler;

		public PREPPopup()
		{
			reuseHandler = true;
		}

		/// <summary>
		/// Property to indicate class reuse state
		/// Part of IHttpHandler
		/// </summary>
		public bool IsReusable
		{
			get
			{
				return reuseHandler;
			}
		}

		/// <summary>
		/// Process the HTTP request called by the application process
		/// Part of IHttpHandler.
		/// This Handler uploads and downloads files
		/// </summary>
		/// <param name="context"></param>
		public void ProcessRequest(HttpContext context)
		{
			string fileName = "";

			try
			{
				if(context.Request.QueryString["fileName"] != null)
				{
					fileName = context.Request.QueryString["fileName"].ToString();
					SPWeb PREPWeb = SPControl.GetContextWeb(context);
					context.Response.Write(PREPPopupHTML(context, fileName));
				}
				
			}
			catch (Exception ex)
			{
				Utilities.LogError("Error in PREPPopup. fileName="+fileName, ex);
			}

		}

		/// <summary>
		/// Build the HTML that will be output as the UI
		/// </summary>
		/// <returns></returns>
		private string PREPPopupHTML(HttpContext context, string fileName)
		{
			SPWeb PREPWeb = SPControl.GetContextWeb(context);

			// The SPFolders representation of the document library has no spaces, whereas the SPList version does.
			// i.e. SPFolders representation = "PREPDocuments"; SPList representation = "PREP Documents". Hence the Replace function below.
			// This is an artifact of reconstituting a site from a template; it won't happen if you build the library manually.
			SPFolder docLibFolder = PREPWeb.Folders[ConfigurationSettings.AppSettings["PREPWorkflow.DocLibrary"].Replace(" ", "")];
			SPFileCollection docLibFiles = docLibFolder.Files;
			SPFile PREPFile = docLibFiles[fileName];

			string authorUserLogin = Utilities.GetLoginName(PREPFile.Author.LoginName);
			string loggedInUserLogin = Utilities.GetLoginName(context.User.Identity.Name);
			string checkedOutToUserLogin = "";
			string docImage = "";
			string altTag = "";
			string statusMsg = "";
			bool bAllowCheckout = false;

			// if file is not checked out, allow download or checkout
			if(PREPFile.CheckOutStatus == SPFile.SPCheckOutStatus.None)
			{
				docImage = "unlockedDoc.gif";
				altTag = fileName + ": " + authorUserLogin + "'s ideas" ;
				statusMsg = "Available for check out.";
				// allow checkout only if the logged-in user is not the author
				if(authorUserLogin != loggedInUserLogin)
				{
					bAllowCheckout = true;
				}
			}
				// if file is checked out, allow download only. 
			else
			{
				docImage = "lockedDoc.gif";
				checkedOutToUserLogin = Utilities.GetLoginName(PREPFile.CheckedOutBy.LoginName);
				statusMsg = "Currently checked out to " + checkedOutToUserLogin;
				altTag = fileName + ": " + authorUserLogin + "'s ideas. Checked out to " + checkedOutToUserLogin;
			}


			StringBuilder pageText = new StringBuilder();

			pageText.Append(HTMLTools.StartPage());
			
			// Page Head
			pageText.Append(HTMLTools.StartHead());
			pageText.Append(HTMLTools.Title("PREP Download"));
			// Styles
			pageText.Append(HTMLTools.StartStyle());
			pageText.Append(HTMLTools.Literal(".Head   {"));
			pageText.Append(HTMLTools.Literal("font-family: Verdana, Helvetica, sans-serif;"));
			pageText.Append(HTMLTools.Literal("font-size: 20px;"));
			pageText.Append(HTMLTools.Literal("font-weight: normal;"));
			pageText.Append(HTMLTools.Literal("color: #666644;"));
			pageText.Append(HTMLTools.Literal("}"));
			pageText.Append(HTMLTools.EndStyle());

			pageText.Append(HTMLTools.EndHead());

			// Body
			pageText.Append(HTMLTools.StartBody());

			// Form
			pageText.Append(HTMLTools.StartForm("post", PREPWeb.Url + "/_layouts/PREPCheckinCheckout.aspx"));
			
			// Outer table
			pageText.Append(HTMLTools.Comment("Outer table"));
			pageText.Append(HTMLTools.StartTable("width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\""));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol());
			pageText.Append(HTMLTools.Break());

			// Inner table
			pageText.Append(HTMLTools.StartTable("width=\"98%\" cellspacing=\"0\" cellpadding=\"4\" border=\"0\""));
			pageText.Append(HTMLTools.StartRow("valign=\"top\""));
			pageText.Append(HTMLTools.StartCol("width=\"20\""));
			pageText.Append(HTMLTools.Literal("&nbsp;"));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.StartCol());
			
			// Heading table
			pageText.Append(HTMLTools.Comment("Heading table"));
			pageText.Append(HTMLTools.StartTable("width=\"300\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\""));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("align=\"left\" class=\"Head\""));
			pageText.Append(HTMLTools.Literal("Get Ideas Document:"));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("colspan=\"2\""));
			pageText.Append(HTMLTools.HorozontalRule("noshade size=\"1\""));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());
			pageText.Append(HTMLTools.EndTable());
			// End Heading Table

			// Image
			pageText.Append(HTMLTools.Image(docImage, altTag));
			
			
			// Document table
			pageText.Append(HTMLTools.StartTable());
			
			// Document title row
			pageText.Append(HTMLTools.Comment("Document title row"));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("valign=\"top\" width=\"70\""));
			pageText.Append(HTMLTools.Literal("Ideas Title:"));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.StartCol());
			pageText.Append(HTMLTools.StartSpan("style=\"font-weight:bold;\""));
			pageText.Append(PREPFile.Title);
			pageText.Append(HTMLTools.EndSpan());
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());

			// Author login row
			pageText.Append(HTMLTools.Comment("Author login row"));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("valign=\"top\""));
			pageText.Append(HTMLTools.Literal("Started By:"));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.StartCol());
			pageText.Append(HTMLTools.StartSpan("style=\"font-weight:bold;\""));
			pageText.Append(authorUserLogin);
			pageText.Append(HTMLTools.EndSpan());
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());

			// Status row
			pageText.Append(HTMLTools.Comment("Status row"));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("valign=\"top\""));
			pageText.Append(HTMLTools.Literal("Status:"));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.StartCol());
			pageText.Append(HTMLTools.StartSpan("style=\"font-weight:bold;\""));
			pageText.Append(HTMLTools.Literal(statusMsg));
			pageText.Append(HTMLTools.EndSpan());
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());

			// Action row (Radio Buttons)
			pageText.Append(HTMLTools.Comment("Action row"));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("valign=\"top\""));
			pageText.Append(HTMLTools.Literal("Action:"));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.StartCol());
			pageText.Append(HTMLTools.Input("type=\"radio\" name=\"prepAction\" value=\"download\" checked=\"checked\""));
			pageText.Append(HTMLTools.Literal("Preview these ideas."));
			pageText.Append(HTMLTools.Break());
			if(bAllowCheckout)
			{
				pageText.Append(HTMLTools.Input("type=\"radio\" name=\"prepAction\" value=\"checkout\""));
				pageText.Append(HTMLTools.Literal("Review and comment on these ideas."));
			}
			pageText.Append(HTMLTools.EndSpan());
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());

			// Submit button row
			pageText.Append(HTMLTools.Comment("Submit button row"));
			pageText.Append(HTMLTools.StartRow());
			pageText.Append(HTMLTools.StartCol("valign=\"top\" colspan=\"2\""));
			pageText.Append(HTMLTools.Input("type=\"submit\" name=\"btnGetFile\" value=\"Get Ideas\" OnClick=\"JavaScript:opener.location.reload();\""));
			pageText.Append(HTMLTools.Input("type=\"hidden\" name=\"fileName\" value=\"" + fileName + "\""));
			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());

			// End Document Table
			pageText.Append(HTMLTools.EndTable());

			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());

			// End Inner table
			pageText.Append(HTMLTools.EndTable());

			pageText.Append(HTMLTools.EndCol());
			pageText.Append(HTMLTools.EndRow());
			
			// End Outer Table
			pageText.Append(HTMLTools.EndTable());

			// End Form
			pageText.Append(HTMLTools.EndForm());
			
			pageText.Append(HTMLTools.EndBody());
			pageText.Append(HTMLTools.EndPage());

			return pageText.ToString();
		}
	}
}
